/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * This is an example program that makes use of a virtual POSIX-like interface
 * and an embedding of that interface inside some possibly-larger system.
 *
 * This program prints out the inner and outer environments.
 */

#define __STRICT_ANSI__ 1

#include <vrpsx_embedding_main.h>
#include <vrpsx_embedding_main_identifier.h>
#include <vrpsx_identified_function.h>
#include <vrpsx_identified_function_identifier.h>
#include <vrpsx_utility.h>
#include <string.h>


VRPSX_DECLARE_IDENTIFIED_FUNCTION(static, main, vrpsx_embedding_main);

static int VRPSX_IDENTIFIED_FUNCTION_NAME(main, vrpsx_embedding_main)(
    vrpsx_embedding_initial_t *initial)
{
    vrpsx_system_t *system = &initial->embedding->system;
    char const *const *ep;
    vrpsx_errno_t ern = 0;

    VRPSX_CALL4(system, write, 1, "INNER:\n", 7, NULL);
    for (ep = initial->environment->array; *ep; ++ep) {
        VRPSX_CALL4(system, write, 1, *ep, strlen(*ep), NULL);
        VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
    }
    VRPSX_CALL4(system, write, 1, "\n", 1, NULL);

    VRPSX_CALL4(system, write, 1, "OUTER:\n", 7, NULL);
    for (ep = initial->outer_environment->array; *ep; ++ep) {
        VRPSX_CALL4(system, write, 1, *ep, strlen(*ep), NULL);
        VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
    }
    VRPSX_CALL4(system, write, 1, "\n", 1, NULL);

    return ern
            ? VRPSX_CONSTANT(system, exit_failure)
            : VRPSX_CONSTANT(system, exit_success);
}

vrpsx_identified_function_t const VRPSX_IDENTIFIED_FUNCTION_LIST[] = {
    VRPSX_MENTION_IDENTIFIED_FUNCTION(main, vrpsx_embedding_main),
    VRPSX_IDENTIFIED_FUNCTION_LIST_TERMINATOR
};
