/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Defines vrpsx_argv_t; see its comments.
 */

#ifndef VRPSX_ARGV_H
#define VRPSX_ARGV_H 1

#include "vrpsx_reference_counted.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#if defined(_MSC_VER)
#pragma warning ( push )
#pragma warning ( disable : 4510 ) /* We do not need a default constructor. */
#pragma warning ( disable : 4610 ) /* We do not need a default constructor. */
#endif /* defined(_MSC_VER) */

/* Lifetime control for an argument array or environment assignment array.
 * The array is terminated by NULL and the count of its non-NULL elements is
 * provided as a convenience.  Each element points to a NUL-terminated string.
 *
 * The text should not be modified until the reference count drops to 0,
 * not even by the implementation.  In particular, environment modification
 * must NOT affect the copy of the data provided by this interface.
 * Dropping this requirement could harm thread safety.
 *
 * Extends: vrpsx_reference_counted_t.
 * Interface Lifetime: reference counted.
 * Thread Safety: thread-safe; constant except for reference count.
 */
typedef struct vrpsx_argv_struct {
    VRPSX_REFERENCE_COUNTED_MEMBERS(struct vrpsx_argv_vtbl_struct)

    /* The array of strings, terminated by NULL.
     * Each string is NUL-terminated.
     */
    char const *const *const array;

    /* The number of non-NULL strings in "array".
     */
    int const dimension;
} vrpsx_argv_t;

#if defined(_MSC_VER)
#pragma warning ( pop )
#endif /* defined(_MSC_VER) */

typedef struct vrpsx_argv_end_struct {
} vrpsx_argv_end_t;

typedef struct vrpsx_argv_vtbl_struct {
    VRPSX_REFERENCE_COUNTED_VTBL_MEMBERS(vrpsx_argv_t);

    /* End marker for documentation and to help detect initialization bugs.
     */
    vrpsx_argv_end_t const *argv_end;
} vrpsx_argv_vtbl_t;


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_ARGV_H */
